#region Dependencies
using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using System.Diagnostics;
using Microsoft.Xna.Framework.Audio;
using System.IO;
#endregion

namespace Spacewar
{
    /// <summary>
    /// SpacewarGame ship
    /// </summary>
    public class Agent1 : Ship
    {
        public bool humanPlay = true;
        private bool avoidSun = false;
        private bool goHyper = false;
        private int agentStart = 1; //0 no, 1 start, 2 after start
        private bool tryTurnLeft = false;
        private bool tryTurnRight = false;
        private bool goForward = false;
        private bool goBackward = false;
        private DateTime fireTime = DateTime.Now;
        private TimeSpan fireInterval = new TimeSpan(0, 0, 0, 0, 100);

    public Agent1(Game game, PlayerIndex player, Vector3 initialPosition, Projectiles bullets)
        : base(game, player, initialPosition, bullets)
        {
            this.player = player;
            this.bullets = bullets;
            this.evolved = false;

            //Scale the ship to match the screen
            scale = new Vector3(4, 8, 8);
        }

        private float calcDistance(float X1, float Y1, float X2, float Y2)
        {
            float XDist = X1 - X2;
            float YDist = Y1 - Y2;
            return (float)Math.Sqrt((XDist * XDist) + (YDist * YDist));
        }
        private float nearestBullet()
        {
            Ship otherShip = Program.getShip1();
            Projectiles otherBullets = otherShip.getBullets();
            float nearestDistance = 1000000.0F;
                
            if (otherBullets.Count > 0)
            {
                for (int bullet=0; bullet < otherBullets.Count; bullet++) {
                    float XBullet = otherBullets[bullet].Position.X;
                    float YBullet = otherBullets[bullet].Position.Y;
                    float XSelf = Position.X;
                    float YSelf = Position.Y;
                    //float XDist = otherBullets[bullet].Position.X - Position.X;
                    //float YDist = otherBullets[bullet].Position.Y - Position.Y;
                    //float distance = (float)Math.Sqrt((XDist * XDist) + (YDist * YDist));
                    float distance = calcDistance(XBullet, YBullet, XSelf, YSelf);
                    if (distance < nearestDistance)
                    {
                        nearestDistance = distance;
                    }
                }
            }
            
            return (nearestDistance);
        }

        private void writeOutputToClips()
        {
            string path = @"c:\research\other\outreach\fromSpaceWars.txt";

            Ship otherShip = Program.getShip1();
            Vector3 ship1Position = otherShip.Position;

            // Open the file to read from.
            using (StreamWriter sw = new StreamWriter(path))
            {
                sw.Write("ship1 position x:");
                sw.Write(ship1Position.X);
                sw.Write(" y:");
                sw.WriteLine(ship1Position.Y);
                
                sw.Write("ship2 position x:");
                sw.Write(Position.X);
                sw.Write(" y:");
                sw.WriteLine(Position.Y);
                
                sw.WriteLine(nearestBullet());
            }
        }

       private float turnRate = 0.05F;
       private float turnLeft(float currentAngle)
       {   
           //GamePads gamePads = XInputHelper.gamePads;            
           return currentAngle + turnRate; }

       private float turnRight(float currentAngle)
       { return currentAngle - turnRate; }

       
        private void pointAwayFromSun()
        {
            float goalAngle = (float)Math.Atan2(position.Y, position.X);  //assume sun at 0,0
            if (goalAngle > -1.57F) goalAngle = goalAngle - 1.57F;
            else goalAngle = goalAngle + 4.71F;
            float currentAngle = Rotation.Z;
            if (currentAngle < (goalAngle-0.1F))
                currentAngle = turnLeft(currentAngle);
            else if (currentAngle > (goalAngle + 0.1F))
                currentAngle = turnRight(currentAngle);
            else
              {
                  Vector3 currentDirection = position;
                  if (currentDirection.Length() < 220.0F)
                  {
                      currentDirection.Normalize();
                      acceleration = acceleration + (currentDirection * 200.0F);
                  }
              }

            Vector3 goalRotation = Rotation;
            goalRotation.Z = currentAngle;
            Rotation = goalRotation; 
        }

        //return true if the other ship has a bullet extant
        private bool isBulletNear()
        {
            /* It seems bullets are duplicated and not attached to a ship.  
               So, it merely returns if there's a bullet about.
            Ship otherShip;
            if (player == PlayerIndex.Two) 
               otherShip = Program.getShip1();
            else
               otherShip = Program.getShip2();
            Projectiles otherBullets = otherShip.getBullets();
             
             */
            Projectiles otherBullets = getBullets();
            
            if (otherBullets.Count > 0)
            {
                //double temp = nearestBullet();
                return true;
            }

            return false;
        }

        private bool nearSun()
        {
            float X = position.X;
            float Y = position.Y;
            if (((X * X) + (Y * Y)) < 1000.0)
                return true;
            return false;
        }
        
        //Set facts from the environment for Clips
        private void setClipsFact()
        {
            if (player == PlayerIndex.Two) 
              Program.cppMain(3); //load and reset agent2
            else 
              Program.cppMain(2); //load and reset agent1

            if (isBulletNear()) 
               Program.cppMain(8); //bullet present
            if (nearSun())
               Program.cppMain(5); //near sun
            if ((Program.rand.NextDouble()%1) < 0.1)
              Program.cppMain(6); //Random 10
            if ((Program.rand.NextDouble()%1) < 0.5)
              Program.cppMain(7); //Random 10
            if (agentStart == 1)
            {
              Program.cppMain(9); //start
              agentStart = 2;
            }

            Program.cppMain(4); //run clips
        }

        private void resetFactsFromClips()
        {
            goHyper = false;
            avoidSun = false;
            fire = false;
            tryTurnLeft = false;
            tryTurnRight = false;
            goForward = false;
            goBackward = false;
        }
        private void getClipsFact()
        {
            resetFactsFromClips();
            //0 call this first to get the clips facts in a file
            //1 avoid sun, 2 fire, 3 left
            //4 goHyper, 5 right, 6 forward, 7 back
            int i;
            i = Program.cppGetFact(0);
            
            i = Program.cppGetFact(1);
            if (i == 1) avoidSun = true;
            
            i = Program.cppGetFact(2);
            if (i == 1)
            {
                DateTime curTime = DateTime.Now;
                TimeSpan timeSinceLastFire = curTime - fireTime;
                if (timeSinceLastFire >= fireInterval)
                {
                    fire = true;
                    fireTime = DateTime.Now;
                }
            }

            i = Program.cppGetFact(3);
            if (i == 1) 
                tryTurnLeft = true;
            
            i = Program.cppGetFact(4);
            if (i == 1) goHyper = true;

            i = Program.cppGetFact(5);
            if (i == 1)
            {  //can't turn both left and right
                if (tryTurnLeft)
                    tryTurnLeft = false; 
                else tryTurnRight = true;
            }
            i = Program.cppGetFact(6);
            if (i == 1) 
                goForward = true;

            i = Program.cppGetFact(7);
            if (i == 1)
            {  //can't move forward and back
                if (goForward)
                    goForward = false;
                else goBackward = true;
            }
            
        }

        private void act(TimeSpan time)
        {
            if (avoidSun)
            {
                pointAwayFromSun();
            }
            if (goHyper)
            {             
                inHyperspace = true;
                inRecovery = false;
                goHyper = false;
            }
            if (tryTurnLeft)
            {
                Vector3 goalRotation = Rotation; 
                goalRotation.Z = turnLeft(Rotation.Z);
                Rotation = goalRotation;   
            }
            else if (tryTurnRight)
            {
                Vector3 goalRotation = Rotation;
                goalRotation.Z = turnRight(Rotation.Z);
                Rotation = goalRotation;
            }
            if (goForward)
            {
                Vector2 thrustDirection = new Vector2(
                    (float)(-SpacewarGame.Settings.ThrustPower * Math.Sin(Rotation.Z)), 
                    (float)(SpacewarGame.Settings.ThrustPower * Math.Cos(Rotation.Z)));
                acceleration += new Vector3(thrustDirection.X,
                    thrustDirection.Y, 0);
            }
            else if (goBackward)
            {
                Vector2 thrustDirection = new Vector2(
                    (float)(-SpacewarGame.Settings.ThrustPower * Math.Sin(Rotation.Z)),
                    (float)(SpacewarGame.Settings.ThrustPower * Math.Cos(Rotation.Z)));
                acceleration -= new Vector3(thrustDirection.X,
                    thrustDirection.Y, 0);
            }

            //if (fire) fireBullet(time); done in ship.update
        }
        public override void Update(TimeSpan time, TimeSpan elapsedTime)
        {
            if (!humanPlay)
            {
                if (inRecovery && (agentStart == 0)) 
                  agentStart = 1;
                if (!inRecovery && (agentStart == 2))
                  agentStart = 0;

                //set clips fact and run clips
                setClipsFact();

                //get clips data
                getClipsFact();

                act(time);
            }
            //let the ship do everything that we don't. 
            //Note acceleration zeroing removed from base.
            base.Update(time, elapsedTime);
          
        }
       
                       
               
    }
}
